! WHIZARD 1.13 Aug 06 2001
! 
! (C) 1999-2001 by Wolfgang Kilian <kilian@particle.uni-karlsruhe.de>
!
! WHIZARD is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! WHIZARD is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! This file has been stripped of most comments.  For documentation, refer
! to the source 'whizard.nw'

module diagnostics

  use limits, only: BUFFER_SIZE, WHIZARD_ROOT, MAX_ERRORS
  use mpi90, only: mpi90_rank

  implicit none
  private

  public :: msg_count_clear
  public :: msg_summary
  public :: msg_terminate
  public :: msg_bug, msg_fatal, msg_error, msg_warning
  public :: msg_message, msg_result, msg_debug

  integer, parameter :: &
       & TERMINATE=-2, BUG=-1, &
       & FATAL=0, ERROR=1, WARNING=2, MESSAGE=3, RESULT=4, DEBUG=5

  integer, public, save :: msg_level = RESULT
  integer, dimension(0:5), public, save :: msg_count = 0
  character(len=BUFFER_SIZE), public, save :: msg_buffer = " "

contains

  subroutine msg_count_clear
    msg_count = 0
  end subroutine msg_count_clear

  subroutine msg_summary (unit)
    integer, intent(in), optional :: unit
1   format (1x,A,1x,I2,1x,A,I2,1x,A)
    if (msg_count(ERROR) > 0 .and. msg_count(WARNING) > 0) then
       write (msg_buffer, 1) "There were", &
            & msg_count(ERROR), "error(s) and", &
            & msg_count(WARNING), "warning(s)."
       call msg_message (unit=unit)
    else if (msg_count(ERROR) > 0) then
       write (msg_buffer, 1) "There were", &
            & msg_count(ERROR), "error(s) and no warnings."
       call msg_message (unit=unit)
    else if (msg_count(WARNING) > 0) then
       write (msg_buffer, 1) "There were no errors and", &
            & msg_count(WARNING), "warning(s)."
       call msg_message (unit=unit)
    end if
  end subroutine msg_summary

  subroutine buffer_clear
    msg_buffer = " "
  end subroutine buffer_clear

  subroutine message_print (level, string, unit)
    integer, intent(in) :: level
    character(len=*), intent(in), optional :: string
    integer, intent(in), optional :: unit
    character(len=80) :: prep_string
    integer :: proc_id
    call mpi90_rank (proc_id)
    if (proc_id==WHIZARD_ROOT) then
       select case (level)
       case (TERMINATE)
          prep_string = " !"
       case (BUG)
          prep_string = " ! *** WHIZARD bug:"
       case (FATAL)
          prep_string = " ! *** Fatal error:"
       case (ERROR)
          prep_string = " ! *** Error:"
       case (WARNING)
          prep_string = " ! Warning:"
       case (MESSAGE, DEBUG)
          prep_string = " !"
       case default
          prep_string = " "
       end select
       if (present(string))  msg_buffer = string
       if (present(unit)) then
          if (unit/=6) then
             write (unit, "(A,A)") trim(prep_string), trim(msg_buffer)
          else if (level <= msg_level) then
             print "(A,A)", trim(prep_string), trim(msg_buffer)
             msg_count(level) = msg_count(level) + 1
          end if
       else if (level <= msg_level) then
          print "(A,A)", trim(prep_string), trim(msg_buffer)
          msg_count(level) = msg_count(level) + 1
       end if
    end if
    call buffer_clear
  end subroutine message_print

  subroutine msg_terminate (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (TERMINATE, string, unit)
    call msg_summary (unit)
    call message_print (MESSAGE, " WHIZARD run finished.", unit)
    stop
  end subroutine msg_terminate

  subroutine msg_bug (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (BUG, string, unit)
    stop "WHIZARD run aborted."
  end subroutine msg_bug

  subroutine msg_fatal (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (FATAL, string, unit)
    stop "WHIZARD run aborted."
  end subroutine msg_fatal

  subroutine msg_error (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (ERROR, string, unit)
    if (msg_count(ERROR) >= MAX_ERRORS) then
       call msg_fatal (" Too many errors encountered.")
    else if (.not.present(unit))  then
       call message_print (MESSAGE, "            (WHIZARD run continues)")
    end if
  end subroutine msg_error

  subroutine msg_warning (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (WARNING, string, unit)
  end subroutine msg_warning

  subroutine msg_message (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (MESSAGE, string, unit)
  end subroutine msg_message

  subroutine msg_result (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (RESULT, string, unit)
  end subroutine msg_result

  subroutine msg_debug (string, unit)
    integer, intent(in), optional :: unit
    character(len=*), intent(in), optional :: string
    call message_print (DEBUG, string, unit)
  end subroutine msg_debug


end module diagnostics
